import { Button, Col, Divider, Modal, Row, Select, Space, Spin } from "antd";
import { useDicomUploader } from "./DicomUploader";
import { Series, Study } from "./DicomUploader/util";
import { DicomTable } from "./DicomTable";
import { useDomain } from "@/hook/useDomain";
import { Typography } from "antd";
import { useState } from "react";
import { User } from "@@/domain/User/entities/User";
import { limitConcurrency } from "./limitConcurrency";
import { useProgress } from "./useProgress";

const { Text } = Typography;

interface DicomUploadProps {
  children?: JSX.Element;
}

/**
 * 新窗口打开ohif阅片
 */
const openOHIFViewer = (
  StudyInstanceUID: string,
  SeriesInstanceUID: string
) => {
  const target = `http://localhost:3000/viewer/${StudyInstanceUID}?SeriesInstanceUID=${SeriesInstanceUID}`;
  window.open(target, "_blank");
};

export const DicomUpload = (props: DicomUploadProps) => {
  const { UploadInput, fileCalculator, studys, isLoading } = useDicomUploader();
  const { ProgressModal, showProgress, hideProgress, updatePercent } =
    useProgress();
  const { dicomDomainService, userDomainService } = useDomain();
  const { dcmFileNum, totalFileNum, dcmFileSize } = fileCalculator;
  const [selectRows, setSelectedRows] = useState<Study[]>([]);
  const [isModalOpen, setIsModalOpen] = useState(false);
  const [annotators, setAnnotators] = useState<User[]>([]);
  const [selectAnnotator, setSelectAnnotator] = useState<User | undefined>(
    undefined
  );

  const onUploadFiles = async (study: Study, series: Series) => {
    const { SeriesInstanceUID, subs } = series;
    const { StudyInstanceUID } = study;
    const instances = await dicomDomainService.existInPacs({
      SeriesInstanceUID,
      StudyInstanceUID,
    });
    // pacs已存在
    if (instances.length === subs.length) {
      openOHIFViewer(StudyInstanceUID, SeriesInstanceUID);
    } else {
      const uploadFunc = (f: File) => () => dicomDomainService.upload2Pacs(f);
      showProgress();
      limitConcurrency(
        series.subs.map(uploadFunc),
        10,
        (completed, total) => {
          console.log(`${completed} out of ${total} tasks completed.`);
          updatePercent(Math.floor((completed / total) * 100));
        },
        () => hideProgress()
      );
    }
  };

  const onClickAssign = () => {
    userDomainService.getDmpAnnotators().then((res) => {
      setIsModalOpen(true);
      setAnnotators(res.data as User[]);
    });
  };

  const dcmFileInfo = !!totalFileNum && (
    <Space>
      <Text type="secondary">扫描总文件数: {totalFileNum}</Text>
      <Divider type="vertical" />
      <Text type="secondary">包含: {dcmFileNum} 个dicom文件</Text>
      <Divider type="vertical" />
      <Text type="secondary">dcm文件总体积: {dcmFileSize} </Text>
    </Space>
  );

  /**
   * 分配任务
   */
  const onAssignConfirm = () => {
    if (!selectAnnotator?.id) return;
    userDomainService.assignLabelDicom(selectAnnotator, selectRows);
    setSelectAnnotator(undefined);
    setIsModalOpen(false);
  };

  return (
    <div style={{ padding: 20 }}>
      <Row style={{ paddingBottom: 20 }}>
        <Col span={24}>
          <Space>
            <UploadInput />
            <Button
              disabled={selectRows.length === 0}
              type="primary"
              onClick={onClickAssign}
            >
              分配
            </Button>
          </Space>
        </Col>
      </Row>
      {dcmFileInfo}
      <DicomTable
        studys={studys}
        loading={isLoading}
        onSelectedRows={(rows) => setSelectedRows(rows)}
        onUploadFiles={onUploadFiles}
      />
      <Modal
        width={320}
        title="选择标注"
        open={isModalOpen}
        closable
        cancelText="再想想"
        okText="确定"
        onOk={onAssignConfirm}
        onCancel={() => setIsModalOpen(false)}
      >
        <Select
          style={{ width: "100%", marginBottom: 20 }}
          placeholder="选择标注"
          value={selectAnnotator?.id as number}
          onChange={(id: number) =>
            setSelectAnnotator(annotators.find((a) => a.id === id))
          }
          options={annotators.map((a) => ({
            value: a.id,
            label: a.username,
          }))}
        />
      </Modal>
      <ProgressModal />
    </div>
  );
};
